
# q-expansion of newform 2300.2.a.l, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2300
weight = 2
poly_data = [2, -1, -7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-4, 0, 1, 0], 1], [[-1, -6, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [0, -1, 0, 0], [0, 0, 0, 0], [0, -1, 0, 1], [0, -1, 0, -1], [0, -1, -1, 1], [-2, -2, 0, 0], [2, 1, 0, 1], [1, 0, 0, 0], [-2, 0, -1, -1], [4, -1, 1, 0], [0, -2, 2, 0], [0, -2, -2, 1], [2, 1, 0, 1], [-4, 0, -1, -2], [2, 0, 0, -2], [3, 1, 0, 1], [6, 0, 0, 2], [-4, 0, -2, 0], [4, -2, -1, 0], [2, -2, -1, 1], [6, 3, 0, -1], [2, 3, 0, 1], [0, 2, 2, -4], [-2, 0, -2, 4], [5, -1, 2, -3], [2, 3, 0, -3], [-2, 2, 2, -2], [4, -2, 2, 2], [6, 2, -2, 2], [0, 1, -1, 0], [4, 5, 0, -2], [-2, 4, 0, 0], [8, 2, 1, -4], [2, -6, 2, 4], [0, -2, -5, 0], [-2, 4, 2, 2], [-4, 1, -1, 4], [1, 1, 4, -3], [9, 2, 0, 2], [4, -5, -1, 6], [2, 8, 0, -4], [6, -1, 0, 5], [2, -1, 4, 2], [-8, 4, -2, -3], [6, 3, -2, -1], [-1, -3, 0, -3], [-11, 5, 0, -3], [-4, 2, -8, 0], [-4, -4, 0, -4], [12, 1, 3, -5], [-8, 3, -4, -4], [-8, -8, 2, 2], [-6, 2, -6, 2], [2, 9, 0, -2], [-4, 2, -6, 0], [-4, 5, 1, 3], [3, -15, 0, 5], [4, 8, 2, -3], [-12, 2, -2, -4], [-14, 2, 0, -6], [-2, 6, 0, -4], [16, -2, 4, 2], [-4, -5, 5, -4], [-2, 8, 4, 0], [-12, -3, -2, 3], [4, -7, 4, 0], [8, -4, 2, -4], [9, 1, 4, 1], [12, 5, 1, -5], [4, 6, -2, -1], [-4, 9, -2, 1], [2, 1, -2, -5], [-22, 2, -2, -4], [20, -8, 2, 4], [2, -1, 4, 3], [-10, 10, 0, -4], [0, -3, -1, -4], [-26, 2, -2, -2], [0, -10, 2, 7], [-4, -1, 0, 9], [-4, -4, -2, -4], [4, 8, 2, 4], [-22, -6, 2, 0], [8, 7, 4, -2], [-8, -10, 3, 4], [-7, 7, 2, -7], [-2, -6, 0, -8], [-4, -15, -3, 7], [11, 3, 0, 3], [-4, 3, -4, 1], [-8, -3, 6, -1], [16, -2, -3, -4], [-12, 14, 1, -4], [0, -7, 0, -2], [6, 1, 4, 1], [-10, -1, -8, 6], [-6, -4, 4, 2], [-20, 5, 2, -3], [0, 12, -2, 5], [12, 12, -3, 2], [-18, -2, 0, 4], [18, -3, 2, -3], [-16, -2, 2, 2], [10, -14, -4, 2], [8, 6, -2, 3], [16, 6, -1, -4], [0, 9, 2, -9], [-5, -5, -4, 3], [-28, 7, -3, -8], [-8, 12, 0, -8], [-10, 14, -2, -4], [-16, 8, -2, 0], [20, -2, -2, 4], [0, 13, 0, 3], [-20, 4, 0, 0], [-8, -5, 8, -1], [0, -5, -3, -8], [2, -8, 1, -7], [-2, -3, 10, -1], [2, -2, -4, -2], [-20, 6, -6, 1], [-10, -2, 0, -4], [-4, 13, -1, -4], [17, 5, 4, -7], [0, -14, 2, 10], [-10, -14, -2, 4], [-4, 2, 8, -6], [14, -4, 4, -10], [8, 6, -6, 0], [20, -5, -1, -2], [-20, -5, -6, 9], [12, 9, 6, -13], [-8, 0, -2, -2], [14, 2, 3, -3], [22, -4, 2, 0], [-6, 16, 2, -2], [-2, -1, 2, -9], [-12, -2, -6, 4], [13, -4, 0, 4], [16, 6, -5, 4], [15, 14, -4, 2], [-12, -17, 4, 0], [-8, 9, 4, -7], [17, -2, 8, -2], [22, 5, -4, 1], [-24, -1, 2, 1], [-8, 1, -5, 0], [24, 1, 5, -6], [28, -9, -4, 2], [7, 1, -6, 11], [16, 4, -2, 6], [1, -13, -4, 3], [12, -11, 0, 10], [8, 3, -6, 9], [-10, 9, -8, -5], [-4, 2, -6, 8], [-18, -10, 1, -1], [16, -6, 0, -8], [16, 10, 6, -12], [24, -5, 9, 4], [22, -2, 0, -6], [-4, 0, 9, 6], [22, -5, -8, 3], [12, 6, 6, -12], [-4, 1, -4, 11], [33, 13, 0, -3], [1, 2, -8, -6], [4, 6, -6, -2], [22, -8, 9, 12], [-24, -1, 2, -7], [16, -11, -11, -1], [20, -7, 0, -9], [12, 8, 2, -8], [36, -3, 0, 8], [0, -4, 0, 2], [8, 5, -1, -4], [12, 0, -6, 3], [-10, -9, 4, 3], [24, -10, -6, -2], [-12, -13, -8, 10], [-4, 17, 8, -3], [12, 4, 2, -15], [5, -4, 4, -12], [0, -13, 0, 11], [-12, 4, -4, -8], [1, -5, 10, -7], [-22, -7, 6, -1], [-12, -9, 9, -7], [12, -22, 0, 2], [22, 14, -3, 3], [19, 11, 8, -17], [-2, -5, -4, 11], [-15, 15, -2, -3], [-12, -5, -12, 7], [6, 2, 4, -10], [12, -2, 2, 0], [4, -12, 6, 4], [30, -6, 6, 12], [40, 11, -4, -2], [40, -2, 4, 6], [17, -1, 4, 11], [16, 16, 2, -7], [-2, 12, 0, -2], [-2, 15, -8, -7], [10, -8, 7, 3], [-6, 8, 6, -2], [20, -20, -1, 2], [4, -7, -1, 19], [4, 5, -1, 12], [10, -6, -7, -5], [28, 11, -1, 1], [-20, -3, 2, -5], [-8, 3, 4, -1], [17, 17, 4, -15], [-20, 8, -2, 6], [-24, -7, 0, -2], [-24, -13, 1, 8], [-18, 7, 0, -5], [10, -4, 5, 4], [10, 4, -3, 5], [12, -11, 12, 3], [-2, -32, 3, 12], [32, 1, 8, -5], [-8, -5, -7, 0], [40, 4, 10, -13], [-18, -16, 6, 2], [-24, -7, -4, -4], [12, -4, 16, 4], [-5, -3, -8, 5], [22, -8, 4, -4], [6, 20, -8, -2], [14, 1, -4, 7], [17, 16, -4, -4], [46, -10, 4, 6], [6, 15, 6, -3], [-30, 8, -8, -8], [18, -2, -6, -2], [12, -13, 0, 8], [-4, 5, 4, -12], [-36, 3, 4, 3], [24, -3, 12, 2], [-36, 20, 3, -2], [36, -3, -5, -4], [21, -5, -2, 13], [0, 18, -1, 0], [-8, 7, -8, 8], [22, -4, -2, -2], [8, 1, -2, 1], [-2, 15, 2, -7], [34, 2, 4, 12], [-6, 4, -6, 4], [26, 5, -2, 15], [8, -22, -2, 2], [27, -18, -4, -6], [-20, 2, 5, -8], [-26, -16, 4, -6], [10, 1, 14, -1], [26, 24, -5, -5], [-10, 4, -1, 4], [-14, 4, -2, 2], [-24, -3, -4, -9], [15, 9, -6, 11], [-38, -10, 0, 0], [-2, -4, -4, 4], [2, -1, 10, 11], [2, 16, 12, -10], [4, 26, 0, -14], [2, 1, -4, 13], [32, 0, -6, 1], [22, 12, 3, 7], [-2, 8, -2, -6], [-68, 0, 0, -2], [-4, -7, 0, -19], [36, -1, 1, 13], [40, 7, -3, 8], [13, -5, 0, 15], [11, -6, 8, -18], [-22, 10, 2, 4], [-18, 6, 2, 6], [-20, 5, -1, -8], [16, -2, 2, 8], [42, 11, 10, -19], [22, 10, 0, 2], [-7, -29, 4, -5], [-24, 8, 4, -16], [-4, 10, 8, -14], [-26, -2, 4, -6], [-15, -7, -8, -11], [29, 20, -12, 0], [10, -6, -2, 20], [-14, 1, 0, -9], [21, -16, -12, -4], [-6, -16, 2, 14], [-8, -5, 1, -16], [14, -10, 14, -6], [14, 1, 4, -5], [-30, 8, 9, 5], [-4, 5, 3, 12], [-8, -27, 3, 4], [-16, 2, 4, -4], [4, -10, 8, 10], [14, -11, 0, 7], [-32, -12, 5, -10], [6, 14, 10, -2], [8, 18, -10, -3], [48, -16, -4, -4], [6, 0, 2, -10], [-22, -12, 6, -14], [0, 15, 6, -7], [-8, 12, -1, 2], [42, 0, 8, -6], [-36, 10, 4, 4], [20, -2, 5, 4], [-14, 20, -2, -14], [-48, 6, -12, -10], [-16, -1, -17, 6], [-12, 14, -5, 8], [-28, -2, -12, -8], [-18, 21, -4, 6], [-2, -20, -10, 2], [16, -14, -12, 2], [40, -16, -6, -5], [31, 9, -12, 1], [-42, -8, 2, 4], [-34, 0, -6, -6], [-42, 2, 2, -2], [0, -7, -13, 16], [-18, -5, -2, -5], [0, 8, 12, -10], [-18, -16, -5, 16], [16, -9, 7, -7], [12, 2, -7, 16], [-20, -5, -1, -18], [10, -10, 0, -6], [16, -26, 5, 0], [6, -12, 8, 14], [-46, 16, 2, 8], [-46, 6, 5, 3], [20, 16, -4, 4], [-12, -11, -5, 24], [16, -10, -2, 10], [24, 11, 14, -7], [-30, 16, 10, -6], [26, -12, -2, 16], [-16, -18, 8, -2], [-14, 0, -11, 4], [-17, -11, -16, 17], [4, -17, 12, -9], [-12, -24, -10, 12], [8, -1, 8, -4], [18, -4, 7, 4], [-24, 13, -18, -5], [8, -14, -2, 4], [22, -10, 8, 12], [4, -1, 22, 1], [-66, 1, 0, -5], [52, 21, -8, -1], [-18, -14, -9, 1], [29, -5, 8, -1], [20, -8, 4, 10], [-28, -1, 1, 9], [-32, -1, 4, -2], [26, -7, 14, 3], [8, -6, 21, 0], [10, -12, -5, 16], [8, 5, -21, 0], [-34, 14, 16, 2], [-8, -18, 4, -8], [-20, 5, -4, 4], [9, 17, -4, 13], [4, 7, -8, 24], [-40, 14, 0, 6], [-10, -17, 2, -5], [-6, 16, 9, -15], [12, -20, 1, 10], [20, -1, 8, 21], [-28, 14, -2, -22], [36, -14, -2, 4], [2, -30, 1, 3], [-24, -24, 4, -6], [38, 18, -10, 10], [-29, 13, -4, 5], [14, 21, -4, -14], [-34, 5, -8, -13], [0, 21, -1, 0], [8, -5, 15, 10], [35, 9, 6, -21], [36, -16, -2, -8], [40, -16, 12, 16], [-10, 2, 6, 8], [-31, 26, 4, -6], [12, -4, -9, 2], [-48, 4, 12, 0], [-10, 10, 2, 2], [-10, 34, -2, -14], [-2, -15, 12, 1], [4, -16, -10, -14], [22, -9, 14, -11], [26, 0, 0, 4], [0, 0, -6, 23], [26, -8, 11, -16], [-34, -9, 0, 3], [-14, 14, 0, 4], [10, 4, 9, -12], [12, -23, 5, 6], [2, 22, 6, -20], [-10, -9, 12, -2], [40, -9, -6, 13], [-23, 5, 8, -11], [34, 20, 9, -24], [16, 7, 15, 9], [0, -11, 16, 7], [26, 25, -10, 1], [-62, 2, 8, 0], [30, 28, -1, -5], [-12, 0, 4, -12], [-13, 35, 4, -13], [-28, -4, -10, -12], [-50, 7, -14, -15], [18, -42, -6, 12], [-16, 20, 10, 8], [-1, 3, -4, -17], [61, 1, 16, -3], [-12, 19, -7, 9], [-8, 11, -9, 25], [-28, 19, -8, -7], [-21, -4, 4, -16], [-12, 7, -4, 0], [32, 25, -11, 2]]
